#ifndef EXECUTE_HPP
#define EXECUTE_HPP

#include <string>
#include <map>
#include <optional>

namespace dropshell {

class sCommand;

// mode bitset
enum class cMode {
    Defaults = 0,
    Silent = 1,
    CaptureOutput = 2
};

inline cMode operator&(cMode lhs, cMode rhs) {return static_cast<cMode>(static_cast<int>(lhs) & static_cast<int>(rhs));}
inline cMode operator+(cMode lhs, cMode rhs) {return static_cast<cMode>(static_cast<int>(lhs) | static_cast<int>(rhs));}
inline cMode operator-(cMode lhs, cMode rhs) {return static_cast<cMode>(static_cast<int>(lhs) & ~static_cast<int>(rhs));}
inline cMode operator|(cMode lhs, cMode rhs) {return static_cast<cMode>(static_cast<int>(lhs) | static_cast<int>(rhs));}
inline cMode operator|=(cMode & lhs, cMode rhs) {return lhs = lhs | rhs;}
inline bool hasFlag(cMode mode, cMode flag) {return (mode & flag) == flag;}


typedef struct sSSHInfo {
    std::string host;
    std::string user;
    std::string port;
} sSSHInfo;

bool execute_command(const sSSHInfo ssh_info, const sCommand command, const cMode mode);
bool execute_command(const sSSHInfo ssh_info, const sCommand command, const cMode mode, std::string & output);
bool execute_command(const sCommand command, const cMode mode);
bool execute_command(const sCommand command, const cMode mode, std::string & output);

// ------------------------------------------------------------------------------------------------



// class to hold a command to run on the remote server.
class sCommand {
    public:
        sCommand(std::string directory_to_run_in, const std::vector<std::string> & command_to_run, const std::map<std::string, std::string> & env_vars) : 
            mDir(directory_to_run_in), mCmdArgs(command_to_run), mEnvVars(env_vars) {}
        sCommand(const std::vector<std::string> & command_to_run) : 
            mDir(""), mCmdArgs(command_to_run), mEnvVars({}) {}
        sCommand() : mDir(""), mCmdArgs({}), mEnvVars({}) {}

        std::string get_directory_to_run_in() const { return mDir; }
        const std::vector<std::string>& get_command_to_run() const { return mCmdArgs; }
        const std::map<std::string, std::string>& get_env_vars() const { return mEnvVars; }

        void add_env_var(const std::string& key, const std::string& value) { mEnvVars[key] = value; }
        
        bool empty() const { return mCmdArgs.empty(); }

    private:
        std::string mDir;
        std::vector<std::string> mCmdArgs;
        std::map<std::string, std::string> mEnvVars;
};


} // namespace dropshell



#endif
